/*
 * fwhose - display file information for owner
 *
 * synopsis:
 *	fwhose [-u|-g] [-n|-N] [-L|-P] [-p|-f] [--] file [...]
 *
 * description:
 *	fwhose displays the owner of file. If you want to display more
 *	detail information about files, consider using stat(1)
 *	appeared in NetBSD 1.6.
 *
 * options:
 *	-u|-g	display uid, gid only. default both.
 *	-n|-N	display uid, gid numerically rather than name, and v.v.
 *	-L|-P	use stat(2) instead of lstat(2), and vice versa.
 *	-p|-f	show the name of file prior or following time.
 *
 * Copyright (c) 2007 Taiji Yamada <taiji@aihara.co.jp>
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>
/* not completely the same as BSD strlcpy */
#define strlcpy(d, s, z) do {\
  strncpy(d, s, z);\
  if ((z) > 0)\
    d[(z)-1] = '\0';\
} while (0)

int usage(int rc)
{
  puts("usage: fwhose [-u|-g] [-n|-N] [-L|-P] [-p|-f] [--] file [...]");
  return rc;
}
int main(int argc, char *argv[])
{
  int a, o = (1<<0)|(1<<1), N = !0, l = !0, f = 0, rc = 0;
  struct stat sb;
  char buf[256], ubuf[64] = "", gbuf[64] = "";

  for (a=1; a<argc; a++)
    if (strcmp(argv[a], "--")==0) break;
    else if (strcmp(argv[a], "-u")==0) o = (1<<0);
    else if (strcmp(argv[a], "-g")==0) o = (1<<1);
    else if (strcmp(argv[a], "-n")==0) N = 0;
    else if (strcmp(argv[a], "-N")==0) N = !0;
    else if (strcmp(argv[a], "-L")==0) l = 0;
    else if (strcmp(argv[a], "-P")==0) l = !0;
    else if (strcmp(argv[a], "-p")==0) f = 1;
    else if (strcmp(argv[a], "-f")==0) f = 2;
    else if (strcmp(argv[a], "--help")==0) return usage(0);
    else if (argv[a][0] == '-') return usage(1);
    else break;
  for (; a<argc; a++)
    if ((l?lstat(argv[a], &sb):stat(argv[a], &sb)) == 0) {
      if (o & (1<<0)) {
	if (!N) sprintf(ubuf, "%d", sb.st_uid);
	else strlcpy(ubuf, getpwuid(sb.st_uid)->pw_name, sizeof(ubuf));
      }
      if (o & (1<<1)) {
	if (!N) sprintf(gbuf, "%d", sb.st_gid);
	else strlcpy(gbuf, getgrgid(sb.st_gid)->gr_name, sizeof(gbuf));
      }
      if (o == ((1<<0)|(1<<1)))	sprintf(buf, "%s.%s", ubuf, gbuf);
      else if (o == (1<<0))	sprintf(buf, "%s", ubuf);
      else if (o == (1<<1))	sprintf(buf, "%s", gbuf);
      switch (f) {
      case 0: printf("%s\n", buf); break;
      case 1: printf("%s\t%s\n", argv[a], buf); break;
      case 2: printf("%s\t%s\n", buf, argv[a]); break;
      }
      buf[0] = '\0';
    }
    else rc = 1;
  return rc;
}
